@@ -158,6 +158,13 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         schema = self.schema
         names = list(self.names)
 
+        if name != name.strip():
+            # Someone could have mistakenly added a space at the end
+            # of the input field.
+            LOG.warn("stripped space from new column %r -> %r", name,
+                     name.strip())
+            name = name.strip()
+
         if name in schema:
             raise CatalogError('The column %s already exists' % name)
 
@@ -239,6 +246,13 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         if not name:
             raise CatalogError('Name of index is empty')
 
+        if name != name.strip():
+            # Someone could have mistakenly added a space at the end
+            # of the input field.
+            LOG.warn("stripped space from new index %r -> %r", name,
+                     name.strip())
+            name = name.strip()
+
         indexes = self.indexes
 
         if isinstance(index_type, str):
@@ -456,14 +470,20 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
                     query[iid] = value
         return query
 
+    def _get_index_query_names(self, index):
+        if hasattr(index, 'getIndexQueryNames'):
+            return index.getIndexQueryNames()
+        return (index.getId(),)
+
     def _sorted_search_indexes(self, query):
         # Simple implementation doing no ordering.
         query_keys = query.keys()
         order = []
         for name, index in self.indexes.items():
-            if name not in query_keys:
-                continue
-            order.append((ILimitedResultIndex.providedBy(index), name))
+            for attr in self._get_index_query_names(index):
+                if attr in query_keys:
+                    order.append((ILimitedResultIndex.providedBy(index), name))
+                    break
         order.sort()
         return [i[1] for i in order]
 
@@ -674,8 +694,6 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         _intersection = intersection
         _self__getitem__ = self.__getitem__
         index_key_map = sort_index.documentToKeyMap()
-        _None = None
-        _keyerror = KeyError
         result = []
         append = result.append
         if hasattr(rs, 'keys'):
@@ -730,8 +748,8 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
                 # get a sorted sequence of the intersections.
                 intset = _intersection(rs, intset)
                 if intset:
-                    keys = getattr(intset, 'keys', _None)
-                    if keys is not _None:
+                    keys = getattr(intset, 'keys', None)
+                    if keys is not None:
                         # Is this ever true?
                         intset = keys()
                     length += len(intset)
@@ -750,9 +768,9 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
             for did in rs:
                 try:
                     key = index_key_map[did]
-                except _keyerror:
+                except KeyError:
                     # This document is not in the sort key index, skip it.
-                    pass
+                    actual_result_count -= 1
                 else:
                     append((key, did, _self__getitem__))
                     # The reference back to __getitem__ is used in case
@@ -784,9 +802,9 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
             for did in rs:
                 try:
                     key = index_key_map[did]
-                except _keyerror:
+                except KeyError:
                     # This document is not in the sort key index, skip it.
-                    pass
+                    actual_result_count -= 1
                 else:
                     if n >= limit and key <= worst:
                         continue
@@ -816,9 +834,9 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
             for did in rs:
                 try:
                     key = index_key_map[did]
-                except _keyerror:
+                except KeyError:
                     # This document is not in the sort key index, skip it.
-                    pass
+                    actual_result_count -= 1
                 else:
                     if n >= limit and key >= best:
                         continue
