@@ -50,6 +50,7 @@ class dummy(ExtensionClass.Base):
     att1 = 'att1'
     att2 = 'att2'
     att3 = ['att3']
+    foo = 'foo'
 
     def __init__(self, num):
         self.num = num
@@ -64,6 +65,12 @@ class dummy(ExtensionClass.Base):
         return ['col3']
 
 
+class MultiFieldIndex(FieldIndex):
+
+    def getIndexQueryNames(self):
+        return [self.id, 'bar']
+
+
 class objRS(ExtensionClass.Base):
 
     def __init__(self, num):
@@ -94,6 +101,13 @@ class TestAddDelColumn(CatalogBase, unittest.TestCase):
         from Products.ZCatalog.Catalog import CatalogError
         self.assertRaises(CatalogError, self._catalog.addColumn, '_id')
 
+    def testAddWithSpace(self):
+        self._catalog.addColumn(' space ')
+        self.assertEqual(' space ' not in self._catalog.schema, True,
+                         'space not stripped in add column')
+        self.assertEqual('space' in self._catalog.schema, True,
+                         'stripping space in add column failed')
+
     def testDel(self):
         self._catalog.addColumn('id')
         self._catalog.delColumn('id')
@@ -125,6 +139,17 @@ class TestAddDelIndexes(CatalogBase, unittest.TestCase):
         self.assert_(isinstance(i, type(KeywordIndex('id'))),
                      'add kw index failed')
 
+    def testAddWithSpace(self):
+        idx = KeywordIndex(' space ')
+        self._catalog.addIndex(' space ', idx)
+        self.assertEqual(' space ' not in self._catalog.indexes, True,
+                         'space not stripped in add index')
+        self.assertEqual('space' in self._catalog.indexes, True,
+                         'stripping space in add index failed')
+        i = self._catalog.indexes['space']
+        # Note: i.id still has spaces in it.
+        self.assert_(isinstance(i, KeywordIndex))
+
     def testDelFieldIndex(self):
         idx = FieldIndex('id')
         self._catalog.addIndex('id', idx)
@@ -180,11 +205,13 @@ class TestCatalog(CatalogBase, unittest.TestCase):
                           index_factory=OkapiIndex, lexicon_id='lexicon')
         att3 = KeywordIndex('att3')
         num = FieldIndex('num')
+        foo = MultiFieldIndex('foo')
 
         self._catalog.addIndex('att1', att1)
         self._catalog.addIndex('att2', att2)
         self._catalog.addIndex('att3', att3)
         self._catalog.addIndex('num', num)
+        self._catalog.addIndex('foo', foo)
         self._catalog.addColumn('att1')
         self._catalog.addColumn('att2')
         self._catalog.addColumn('att3')
@@ -285,6 +312,15 @@ class TestCatalog(CatalogBase, unittest.TestCase):
         result = self._catalog._sorted_search_indexes(query)
         self.assertEquals(result.index('att1'), 2)
 
+    def test_sorted_search_indexes_match_alternate_attr(self):
+        query = {'bar': 'b'}
+        result = self._catalog._sorted_search_indexes(query)
+        self.assertEquals(result, ['foo'])
+
+    def test_sorted_search_indexes_no_match(self):
+        result = self._catalog._sorted_search_indexes({'baz': 'a'})
+        self.assertEquals(result, [])
+
     # search
 
     def test_sortResults(self):
@@ -415,6 +451,24 @@ class TestCatalog(CatalogBase, unittest.TestCase):
         self.assertEqual(result.actual_result_count, 100)
         self.assertEqual([r.num for r in result], [])
 
+    def testSortedResultLengthWithMissingDocs(self):
+        # remove the `0` document from the num index only
+        num_index = self._catalog.getIndex('num')
+        pos_of_zero = self.nums.index(0)
+        uid = self._catalog.uids.get(repr(pos_of_zero))
+        self.assertEqual(self._catalog[uid].num, 0)
+        num_index.unindex_object(uid)
+        # make sure it was removed
+        self.assertEqual(len(num_index), 99)
+        # sort over the smaller num index
+        query = dict(att1='att1', sort_on='num', sort_limit=10)
+        result = self._catalog(query)
+        # the `0` document was removed
+        self.assertEqual(result[0].num, 1)
+        self.assertEqual(len(result), 10)
+        # there are only 99 documents left
+        self.assertEqual(result.actual_result_count, 99)
+
     # _get_sort_attr
     # _getSortIndex
     # searchResults
