@@ -123,6 +123,7 @@ class ClientStorage(object):
                  username='', password='', realm=None,
                  blob_dir=None, shared_blob_dir=False,
                  blob_cache_size=None, blob_cache_size_check=10,
+                 client_label=None,
                  ):
         """ClientStorage constructor.
 
@@ -234,12 +235,18 @@ class ClientStorage(object):
             loaded into the cache. Defaults to 10% of the blob cache
             size.   This option is ignored if shared_blob_dir is true.
 
+        client_label
+            A label to include in server log messages for the client.
+
         Note that the authentication protocol is defined by the server
         and is detected by the ClientStorage upon connecting (see
         testConnection() and doAuth() for details).
 
         """
 
+        if isinstance(addr, int):
+            addr = '127.0.0.1', addr
+
         self.__name__ = name or str(addr) # Standard convention for storages
 
         logger.info(
@@ -314,6 +321,8 @@ class ClientStorage(object):
         # _server_addr is used by sortKey()
         self._server_addr = None
 
+        self._client_label = client_label
+
         self._pickler = self._tfile = None
 
         self._info = {'length': 0, 'size': 0, 'name': 'ZEO Client',
@@ -443,15 +452,17 @@ class ClientStorage(object):
                         self.__name__)
 
     def close(self):
-        """Storage API: finalize the storage, releasing external resources."""
-        if self._rpc_mgr is not None:
-            self._rpc_mgr.close()
-            self._rpc_mgr = None
+        "Storage API: finalize the storage, releasing external resources."
+        _rpc_mgr = self._rpc_mgr
+        self._rpc_mgr = None
+        if _rpc_mgr is None:
+            return # already closed
+
         if self._connection is not None:
             self._connection.register_object(None) # Don't call me!
-            self._connection.close()
-            self._connection = None
+        self._connection = None
 
+        _rpc_mgr.close()
         self._tbuf.close()
         if self._cache is not None:
             self._cache.close()
@@ -617,6 +628,15 @@ class ClientStorage(object):
                         self.__name__, self._server_addr)
 
         stub = self.StorageServerStubClass(conn)
+
+        if self._client_label and conn.peer_protocol_version >= "Z310":
+            stub.set_client_label(self._client_label)
+
+        if conn.peer_protocol_version < "Z3101":
+            logger.warning("Old server doesn't suppport "
+                           "checkCurrentSerialInTransaction")
+            self.checkCurrentSerialInTransaction = lambda *args: None
+
         self._oids = []
         self.verify_cache(stub)
 
@@ -919,6 +939,11 @@ class ClientStorage(object):
         self._tbuf.store(oid, data)
         return self._check_serials()
 
+    def checkCurrentSerialInTransaction(self, oid, serial, transaction):
+        self._check_trans(transaction)
+        self._server.checkCurrentSerialInTransaction(oid, serial,
+                                                     id(transaction))
+
     def storeBlob(self, oid, serial, data, blobfilename, version, txn):
         """Storage API: store a blob object."""
         assert not version
@@ -1067,7 +1092,8 @@ class ClientStorage(object):
     def tpc_vote(self, txn):
         """Storage API: vote on a transaction."""
         if txn is not self._transaction:
-            return
+            raise POSException.StorageTransactionError(
+                "tpc_vote called with wrong transaction")
         self._server.vote(id(txn))
         return self._check_serials()
 
@@ -1086,7 +1112,9 @@ class ClientStorage(object):
             # must be ignored.
             if self._transaction == txn:
                 self._tpc_cond.release()
-                return
+                raise POSException.StorageTransactionError(
+                    "Duplicate tpc_begin calls for same transaction")
+
             self._tpc_cond.wait(30)
         self._transaction = txn
         self._tpc_cond.release()
@@ -1140,7 +1168,8 @@ class ClientStorage(object):
     def tpc_finish(self, txn, f=None):
         """Storage API: finish a transaction."""
         if txn is not self._transaction:
-            return
+            raise POSException.StorageTransactionError(
+                "tpc_finish called with wrong transaction")
         self._load_lock.acquire()
         try:
             if self._midtxn_disconnect:
@@ -1235,10 +1264,7 @@ class ClientStorage(object):
 
         """
         self._check_trans(txn)
-        tid, oids = self._server.undo(trans_id, id(txn))
-        for oid in oids:
-            self._tbuf.invalidate(oid)
-        return tid, oids
+        self._server.undoa(trans_id, id(txn))
 
     def undoInfo(self, first=0, last=-20, specification=None):
         """Storage API: return undo information."""
@@ -1313,24 +1339,25 @@ class ClientStorage(object):
         # it should set self._server.  If it goes through full cache
         # verification, then endVerify() should self._server.
 
-        ltid = server.lastTransaction()
+        server_tid = server.lastTransaction()
         if not self._cache:
             logger.info("%s No verification necessary -- empty cache",
                         self.__name__)
-            if ltid and ltid != utils.z64:
-                self._cache.setLastTid(ltid)
+            if server_tid != utils.z64:
+                self._cache.setLastTid(server_tid)
             self.finish_verification()
             return "empty cache"
 
-        last_inval_tid = self._cache.getLastTid()
-        if last_inval_tid is not None:
-            if ltid == last_inval_tid:
+        cache_tid = self._cache.getLastTid()
+        if cache_tid != utils.z64:
+            if server_tid == cache_tid:
                 logger.info(
-                    "%s No verification necessary (last_inval_tid up-to-date)",
-                    self.__name__)
+                    "%s No verification necessary"
+                    " (cache_tid up-to-date %r)",
+                    self.__name__, server_tid)
                 self.finish_verification()
                 return "no verification"
-            elif ltid < last_inval_tid:
+            elif server_tid < cache_tid:
                 message = ("%s Client has seen newer transactions than server!"
                            % self.__name__)
                 logger.critical(message)
@@ -1338,24 +1365,25 @@ class ClientStorage(object):
 
             # log some hints about last transaction
             logger.info("%s last inval tid: %r %s\n",
-                        self.__name__, last_inval_tid,
-                        tid2time(last_inval_tid))
+                        self.__name__, cache_tid,
+                        tid2time(cache_tid))
             logger.info("%s last transaction: %r %s",
-                        self.__name__, ltid, ltid and tid2time(ltid))
+                        self.__name__, server_tid,
+                        server_tid and tid2time(server_tid))
 
-            pair = server.getInvalidations(last_inval_tid)
+            pair = server.getInvalidations(cache_tid)
             if pair is not None:
                 logger.info("%s Recovering %d invalidations",
                             self.__name__, len(pair[1]))
                 self.finish_verification(pair)
                 return "quick verification"
-        elif ltid and ltid != utils.z64:
+        elif server_tid != utils.z64:
 
-            # XXX Hm, to have gotten here, the cache is non-empty, but
+            # Hm, to have gotten here, the cache is non-empty, but
             # it has no last tid. This doesn't seem like good situation.
-            # We shouldn't treat it so lightly.
+            # We'll have to verify the cache, if we're willing.
+            self._cache.setLastTid(server_tid)
 
-            self._cache.setLastTid(ltid)
 
         zope.event.notify(ZEO.interfaces.StaleCache(self))
 
@@ -1371,8 +1399,8 @@ class ClientStorage(object):
         if self._cache and self._drop_cache_rather_verify:
             logger.critical("%s dropping stale cache", self.__name__)
             self._cache.clear()
-            if ltid:
-                self._cache.setLastTid(ltid)
+            if server_tid:
+                self._cache.setLastTid(server_tid)
             self.finish_verification()
             return "cache dropped"
 
@@ -1419,6 +1447,7 @@ class ClientStorage(object):
             min_tid = self._cache.getLastTid()
             while 1:
                 tid, oids = unpickler.load()
+                logger.debug('pickled inval %r %r', tid, min_tid)
                 if oids is None:
                     break
                 if ((tid is None)
@@ -1509,6 +1538,9 @@ class ClientStorage(object):
                 return self._iterator_gc(True)
             self._iterator_ids -= iids
 
+    def server_status(self):
+        return self._server.server_status()
+
 
 class TransactionIterator(object):
 
@@ -1522,7 +1554,7 @@ class TransactionIterator(object):
 
     def next(self):
         if self._ended:
-            raise ZODB.interfaces.StorageStopIteration()
+            raise StopIteration()
 
         if self._iid < 0:
             raise ClientDisconnected("Disconnected iterator")
@@ -1533,7 +1565,7 @@ class TransactionIterator(object):
             # disposed it.
             self._ended = True
             self._storage._forget_iterator(self._iid)
-            raise ZODB.interfaces.StorageStopIteration()
+            raise StopIteration()
 
         return ClientStorageTransactionInformation(
             self._storage, self, *tx_data)
@@ -1574,13 +1606,13 @@ class RecordIterator(object):
         if self._completed:
             # We finished iteration once already and the server can't know
             # about the iteration anymore.
-            raise ZODB.interfaces.StorageStopIteration()
+            raise StopIteration()
         item = self._storage._server.iterator_record_next(self._riid)
         if item is None:
             # The iterator is exhausted, and the server has already
             # disposed it.
             self._completed = True
-            raise ZODB.interfaces.StorageStopIteration()
+            raise StopIteration()
         return ZODB.BaseStorage.DataRecord(*item)
 
 
