@@ -152,7 +152,7 @@ class ClientCache(object):
     # default of 20MB.  The default here is misleading, though, since
     # ClientStorage is the only user of ClientCache, and it always passes an
     # explicit size of its own choosing.
-    def __init__(self, path=None, size=200*1024**2):
+    def __init__(self, path=None, size=200*1024**2, rearrange=.8):
 
         # - `path`:  filepath for the cache file, or None (in which case
         #   a temp file will be created)
@@ -163,6 +163,11 @@ class ClientCache(object):
         size = max(size, ZEC_HEADER_SIZE)
         self.maxsize = size
 
+        # rearrange: if we read a current record and it's more than
+        # rearrange*size from the end, then copy it forward to keep it
+        # from being evicted.
+        self.rearrange = rearrange * size
+
         # The number of records in the cache.
         self._len = 0
 
@@ -177,7 +182,7 @@ class ClientCache(object):
 
         # tid for the most recent transaction we know about.  This is also
         # stored near the start of the file.
-        self.tid = None
+        self.tid = z64
 
         # Always the offset into the file of the start of a block.
         # New and relocated objects are always written starting at
@@ -397,10 +402,13 @@ class ClientCache(object):
     # Close the underlying file.  No methods accessing the cache should be
     # used after this.
     def close(self):
-        if self.f:
-            sync(self.f)
-            self.f.close()
-            self.f = None
+        self._unsetup_trace()
+        f = self.f
+        self.f = None
+        if f is not None:
+            sync(f)
+            f.close()
+
         if hasattr(self,'_lock_file'):
             self._lock_file.close()
 
@@ -455,7 +463,7 @@ class ClientCache(object):
     def setLastTid(self, tid):
         if (not tid) or (tid == z64):
             return
-        if (self.tid is not None) and (tid <= self.tid) and self:
+        if (tid <= self.tid) and self._len:
             if tid == self.tid:
                 return                  # Be a little forgiving
             raise ValueError("new last tid (%s) must be greater than "
@@ -470,13 +478,9 @@ class ClientCache(object):
     ##
     # Return the last transaction seen by the cache.
     # @return a transaction id
-    # @defreturn string, or None if no transaction is yet known
+    # @defreturn string, or 8 nulls if no transaction is yet known
     def getLastTid(self):
-        tid = self.tid
-        if tid == z64:
-            return None
-        else:
-            return tid
+        return self.tid
 
     ##
     # Return the current data record for oid.
@@ -498,17 +502,37 @@ class ClientCache(object):
         size, saved_oid, tid, end_tid, lver, ldata = unpack(
             ">I8s8s8sHI", read(34))
         assert saved_oid == oid, (ofs, self.f.tell(), oid, saved_oid)
+        assert end_tid == z64, (ofs, self.f.tell(), oid, tid, end_tid)
         assert lver == 0, "Versions aren't supported"
 
         data = read(ldata)
         assert len(data) == ldata, (ofs, self.f.tell(), oid, len(data), ldata)
 
         # WARNING: The following assert changes the file position.
-        # We must not depend on ths below or we'll fail in optimized mode.
+        # We must not depend on this below or we'll fail in optimized mode.
         assert read(8) == oid, (ofs, self.f.tell(), oid)
 
         self._n_accesses += 1
         self._trace(0x22, oid, tid, end_tid, ldata)
+
+        ofsofs = self.currentofs - ofs
+        if ofsofs < 0:
+            ofsofs += self.maxsize
+
+        if (ofsofs > self.rearrange and
+            self.maxsize > 10*len(data) and
+            size > 4):
+            # The record is far back and might get evicted, but it's
+            # valuable, so move it forward.
+
+            # Remove fromn old loc:
+            del self.current[oid]
+            self.f.seek(ofs)
+            self.f.write('f'+pack(">I", size))
+
+            # Write to new location:
+            self._store(oid, tid, None, data, size)
+
         return data, tid
 
     ##
@@ -545,7 +569,7 @@ class ClientCache(object):
         assert len(data) == ldata, (ofs, self.f.tell())
 
         # WARNING: The following assert changes the file position.
-        # We must not depend on ths below or we'll fail in optimized mode.
+        # We must not depend on this below or we'll fail in optimized mode.
         assert read(8) == oid, (ofs, self.f.tell(), oid)
 
         if end_tid < before_tid:
@@ -600,6 +624,16 @@ class ClientCache(object):
         self._n_added_bytes += size
         self._len += 1
 
+        self._store(oid, start_tid, end_tid, data, size)
+
+        if end_tid:
+            self._trace(0x54, oid, start_tid, end_tid, dlen=len(data))
+        else:
+            self._trace(0x52, oid, start_tid, dlen=len(data))
+
+    def _store(self, oid, start_tid, end_tid, data, size):
+        # Low-level store used by store and load
+
         # In the next line, we ask for an extra to make sure we always
         # have a free block after the new alocated block.  This free
         # block acts as a ring pointer, so that on restart, we start
@@ -619,6 +653,7 @@ class ClientCache(object):
             extra = 'f' + pack(">I", excess)
 
         ofs = self.currentofs
+        seek = self.f.seek
         seek(ofs)
         write = self.f.write
 
@@ -640,13 +675,12 @@ class ClientCache(object):
 
         if end_tid:
             self._set_noncurrent(oid, start_tid, ofs)
-            self._trace(0x54, oid, start_tid, end_tid, dlen=len(data))
         else:
             self.current[oid] = ofs
-            self._trace(0x52, oid, start_tid, dlen=len(data))
 
         self.currentofs += size
 
+
     ##
     # If `tid` is None,
     # forget all knowledge of `oid`.  (`tid` can be None only for
@@ -661,7 +695,6 @@ class ClientCache(object):
     # - oid object id
     # - tid the id of the transaction that wrote a new revision of oid,
     #        or None to forget all cached info about oid.
-
     @locked
     def invalidate(self, oid, tid):
         ofs = self.current.get(oid)
@@ -738,6 +771,10 @@ class ClientCache(object):
     # self._tracefile.  If not, or we can't write to the trace file, disable
     # tracing by setting self._trace to a dummy function, and set
     # self._tracefile to None.
+    _tracefile = None
+    def _trace(self, *a, **kw):
+        pass
+
     def _setup_trace(self, path):
         _tracefile = None
         if path and os.environ.get("ZEO_CACHE_TRACE"):
@@ -750,7 +787,6 @@ class ClientCache(object):
                 logger.info("opened tracefile %r", tfn)
 
         if _tracefile is None:
-            self._trace = lambda *a, **k: None
             return
 
         now = time.time
@@ -759,7 +795,7 @@ class ClientCache(object):
             # The first hex digit shows the operation, the second the outcome.
             # This method has been carefully tuned to be as fast as possible.
             # Note: when tracing is disabled, this method is hidden by a dummy.
-            encoded = (dlen + 255) & 0x7fffff00 | code
+            encoded = (dlen << 8) + code
             if tid is None:
                 tid = z64
             if end_tid is None:
@@ -774,8 +810,15 @@ class ClientCache(object):
                 raise
 
         self._trace = _trace
+        self._tracefile = _tracefile
         _trace(0x00)
 
+    def _unsetup_trace(self):
+        if self._tracefile is not None:
+            del self._trace
+            self._tracefile.close()
+            del self._tracefile
+
 def sync(f):
     f.flush()
 
