@@ -11,22 +11,21 @@
 #
 ##############################################################################
 """Patch for Persistent to support IPersistentExtra.
-
-$Id: PersistentExtra.py 123713 2011-12-12 10:47:47Z hannosch $
 """
+
 from DateTime.DateTime import DateTime
 
 class PersistentUtil:
 
     def bobobase_modification_time(self):
-        jar=self._p_jar
-        oid=self._p_oid
+        jar = self._p_jar
+        oid = self._p_oid
         if jar is None or oid is None:
             return DateTime()
 
         try:
             t = self._p_mtime
-        except:
+        except AttributeError:
             t = 0
         return DateTime(t)
 
