@@ -21,7 +21,7 @@ def test_InitializeClass():
     Check when the class has an ExtensionClass as attribute.
 
     >>> import ExtensionClass
-    >>> from App.class_init import InitializeClass
+    >>> from AccessControl.class_init import InitializeClass
     >>> class AnotherClass(ExtensionClass.Base):
     ...     _need__name__ = 1
 
@@ -38,6 +38,3 @@ def test_suite():
     return unittest.TestSuite((
         DocTestSuite(),
         ))
-
-if __name__ == '__main__':
-    unittest.main()
