@@ -10,13 +10,13 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""OFS z3 interfaces.
-
-$Id: interfaces.py 113517 2010-06-16 10:53:19Z jens $
+"""OFS interfaces.
 """
+
 from zope.container.interfaces import IContainer
 from zope.interface import Attribute
 from zope.interface import Interface
+from zope.location.interfaces import IRoot
 from zope.schema import Bool, BytesLine, Tuple
 
 from AccessControl.interfaces import IOwned
@@ -28,8 +28,12 @@ from persistent.interfaces import IPersistent
 from webdav.interfaces import IDAVCollection
 from webdav.interfaces import IDAVResource
 
-from zope.traversing.interfaces import IContainmentRoot
-from zope.location.interfaces import IPossibleSite
+# BBB
+try:
+    from zope.component.interfaces import IPossibleSite
+except ImportError:
+    from zope.location.interfaces import IPossibleSite
+
 
 class IOrderedContainer(Interface):
 
@@ -339,7 +343,7 @@ class IItem(IZopeObject, IManageable, IFTPAccess, IDAVResource,
 
     icon = BytesLine(
         title=u"Icon",
-        description=u"Name of icon, relative to SOFTWARE_URL",
+        description=u"Name of icon, relative to BASEPATH1",
         )
 
     def getId():
@@ -834,7 +838,7 @@ class IOrderedFolder(IOrderedContainer, IFolder):
 # XXX: might contain non-API methods and outdated comments;
 #      not synced with ZopeBook API Reference;
 #      based on OFS.Application.Application
-class IApplication(IFolder, IContainmentRoot):
+class IApplication(IFolder, IRoot):
 
     """Top-level system object"""
 
