@@ -11,8 +11,6 @@
 #
 ##############################################################################
 """Traverse unit tests.
-
-$Id: testTraverse.py 110405 2010-04-01 16:13:15Z tseaver $
 """
 
 import unittest
@@ -186,8 +184,7 @@ class TestTraverse( unittest.TestCase ):
         class BoboTraversableWithAcquisition(SimpleItem):
             """ A BoboTraversable which may use acquisition to find objects.
 
-            This is similar to how the __bobo_traverse__ added by Five
-            behaves).
+            This is similar to how the __bobo_traverse__ behaves).
             """
 
             def __bobo_traverse__(self, request, name):
@@ -224,7 +221,7 @@ class TestTraverse( unittest.TestCase ):
         noSecurityManager()
         SecurityManager.setSecurityPolicy(policy)
 
-    def test_z3interfaces(self):
+    def test_interfaces(self):
         from OFS.interfaces import ITraversable
         from OFS.Traversable import Traversable
         from zope.interface.verify import verifyClass
@@ -232,31 +229,31 @@ class TestTraverse( unittest.TestCase ):
         verifyClass(ITraversable, Traversable)
 
     def testTraversePath( self ):
-        self.failUnless( 'file' in self.folder1.objectIds() )
-        self.failUnless(
+        self.assertTrue( 'file' in self.folder1.objectIds() )
+        self.assertTrue(
             self.folder1.unrestrictedTraverse( ('', 'folder1', 'file' ) ))
-        self.failUnless(
+        self.assertTrue(
             self.folder1.unrestrictedTraverse( ('', 'folder1' ) ))
 
     def testTraverseURLNoSlash( self ):
-        self.failUnless( 'file' in self.folder1.objectIds() )
-        self.failUnless(
+        self.assertTrue( 'file' in self.folder1.objectIds() )
+        self.assertTrue(
             self.folder1.unrestrictedTraverse( '/folder1/file' ))
-        self.failUnless(
+        self.assertTrue(
             self.folder1.unrestrictedTraverse( '/folder1' ))
 
     def testTraverseURLSlash( self ):
-        self.failUnless('file' in self.folder1.objectIds())
-        self.failUnless(self.folder1.unrestrictedTraverse( '/folder1/file/'))
-        self.failUnless(self.folder1.unrestrictedTraverse( '/folder1/'))
+        self.assertTrue('file' in self.folder1.objectIds())
+        self.assertTrue(self.folder1.unrestrictedTraverse( '/folder1/file/'))
+        self.assertTrue(self.folder1.unrestrictedTraverse( '/folder1/'))
 
     def testTraverseToNone( self ):
-        self.failUnlessRaises(
+        self.assertRaises(
             KeyError,
             self.folder1.unrestrictedTraverse, ('', 'folder1', 'file2' ) )
-        self.failUnlessRaises(
+        self.assertRaises(
             KeyError, self.folder1.unrestrictedTraverse,  '/folder1/file2' )
-        self.failUnlessRaises(
+        self.assertRaises(
             KeyError, self.folder1.unrestrictedTraverse,  '/folder1/file2/' )
 
     def testTraverseMethodRestricted(self):
@@ -275,14 +272,14 @@ class TestTraverse( unittest.TestCase ):
         # Zope security policy.
         self._setupSecurity()
         bb = self._makeBoboTraversable()
-        self.failUnlessRaises(KeyError, bb.restrictedTraverse, 'notfound')
+        self.assertRaises(KeyError, bb.restrictedTraverse, 'notfound')
         bb.restrictedTraverse('bb_subitem')
 
     def testBoboTraverseToMethod(self):
         # Verify it's possible to use __bobo_traverse__ to a method.
         self._setupSecurity()
         bb = self._makeBoboTraversable()
-        self.failUnless(
+        self.assertTrue(
             bb.restrictedTraverse('bb_method') is not bb.bb_method)
 
     def testBoboTraverseToSimpleAttrValue(self):
@@ -299,7 +296,7 @@ class TestTraverse( unittest.TestCase ):
         # is fine, but to test the code branch we sub in the forgiving one
         self._setupSecurity(UnitTestSecurityPolicy())
         bb = self._makeBoboTraversable()
-        self.failUnless(
+        self.assertTrue(
             bb.restrictedTraverse('manufactured') is 42)
 
     def testBoboTraverseToAcquiredObject(self):
@@ -326,7 +323,7 @@ class TestTraverse( unittest.TestCase ):
         folder.manage_permission(access_contents_information, [], 0)
         bb = self._makeBoboTraversableWithAcquisition()
         bb = bb.__of__(self.root)
-        self.failUnlessRaises(Unauthorized,
+        self.assertRaises(Unauthorized,
                               bb.restrictedTraverse, 'folder1')
 
     def testBoboTraverseToAcquiredAttribute(self):
@@ -352,9 +349,36 @@ class TestTraverse( unittest.TestCase ):
         folder.stuff = 'stuff here'
         bb = self._makeBoboTraversableWithAcquisition()
         bb = bb.__of__(folder)
-        self.failUnlessRaises(Unauthorized,
+        self.assertRaises(Unauthorized,
                               self.root.folder1.restrictedTraverse, 'stuff')
 
+    def testBoboTraverseTraversalDefault(self):
+        from OFS.SimpleItem import SimpleItem
+        from ZPublisher.interfaces import UseTraversalDefault
+        class BoboTraversableUseTraversalDefault(SimpleItem):
+            """
+              A BoboTraversable class which may use "UseTraversalDefault"
+              (dependent on "name") to indicate that standard traversal should
+              be used.
+            """
+            default = 'Default'
+
+            def __bobo_traverse__(self, request, name):
+                if name == 'normal': return 'Normal'
+                raise UseTraversalDefault
+
+
+        bb = BoboTraversableUseTraversalDefault()
+        # normal access -- no traversal default used
+        self.assertEqual(bb.unrestrictedTraverse('normal'), 'Normal')
+        # use traversal default
+        self.assertEqual(bb.unrestrictedTraverse('default'), 'Default')
+        # test traversal default with acqires attribute
+        si = SimpleItem()
+        si.default_acquire = 'Default_Acquire'
+        si.bb = bb
+        self.assertEqual(si.unrestrictedTraverse('bb/default_acquire'), 'Default_Acquire')
+
     def testAcquiredAttributeDenial(self):
         # Verify that restrictedTraverse raises the right kind of exception
         # on denial of access to an acquired attribute.  If it raises
@@ -365,7 +389,7 @@ class TestTraverse( unittest.TestCase ):
         self._setupSecurity(CruelSecurityPolicy())
         newSecurityManager( None, self._makeUser().__of__( self.root ) )
         self.root.stuff = 'stuff here'
-        self.failUnlessRaises(Unauthorized,
+        self.assertRaises(Unauthorized,
                               self.app.folder1.restrictedTraverse, 'stuff')
 
     def testDefaultValueWhenUnathorized(self):
@@ -378,6 +402,20 @@ class TestTraverse( unittest.TestCase ):
         self.assertEqual(
             self.root.folder1.restrictedTraverse('stuff', 42), 42)
 
+    def testNotFoundIsRaised(self):
+        from OFS.SimpleItem import SimpleItem
+        from zExceptions import NotFound
+        from operator import getitem
+        self.folder1._setObject('foo', SimpleItem('foo'))
+        self.assertRaises(AttributeError, getitem, self.folder1.foo,
+                          'doesntexist')
+        self.assertRaises(NotFound, self.folder1.unrestrictedTraverse,
+                          'foo/doesntexist')
+        self.assertRaises(TypeError, getitem,
+                          self.folder1.foo.isPrincipiaFolderish, 'doesntexist')
+        self.assertRaises(NotFound, self.folder1.unrestrictedTraverse,
+                          'foo/isPrincipiaFolderish/doesntexist')
+
     def testDefaultValueWhenNotFound(self):
         # Test that traversing to a non-existent object returns
         # the default when provided
@@ -388,33 +426,27 @@ class TestTraverse( unittest.TestCase ):
     def testTraverseUp(self):
         # Test that we can traverse upwards
         from Acquisition import aq_base
-        self.failUnless(
+        self.assertTrue(
             aq_base(self.root.folder1.file.restrictedTraverse('../..')) is
             aq_base(self.root))
     
     def testTraverseToNameStartingWithPlus(self):
         # Verify it's possible to traverse to a name such as +something
-        self.failUnless(
+        self.assertTrue(
             self.folder1.unrestrictedTraverse('+something') is 'plus')
 
 
-import os, sys
-if __name__ == '__main__':
-    execfile(os.path.join(sys.path[0], 'framework.py'))
-
-
 class SimpleClass(object):
     """Class with no __bobo_traverse__."""
 
 
 def test_traversable():
     """
-    Test the behaviour of unrestrictedTraverse and views. The tests are
-    copies from Five.browser.tests.test_traversable, but instead of
-    publishing they do unrestrictedTraverse.
+    Test the behaviour of unrestrictedTraverse and views. The tests don't
+    use publishing but do unrestrictedTraverse instead.
 
       >>> import Products.Five
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config("configure.zcml", Products.Five)
       >>> from Testing.makerequest import makerequest
       >>> self.app = makerequest(self.app)
@@ -433,9 +465,9 @@ def test_traversable():
       ...    self.folder.testoid.unrestrictedTraverse('doesntexist')
       ... except NotFound:
       ...    pass
-      
+
     Now let's take class which already has a __bobo_traverse__ method.
-    Five should correctly use that as a fallback.
+    We should correctly use that as a fallback.
 
       >>> configure_zcml = '''
       ... <configure xmlns="http://namespaces.zope.org/zope"
@@ -527,7 +559,7 @@ def test_traversable():
 
     Without a __bobo_traverse__ method this would have returned the attribute
     value 'This is an attribute'.  Let's make sure the same thing happens for
-    an object that has been marked traversable by Five:
+    an object that has been marked traversable:
 
       >>> self.folder.fancy.an_attribute = 'This is an attribute'
       >>> self.folder.fancy.unrestrictedTraverse(
@@ -594,7 +626,7 @@ def test_view_doesnt_shadow_attribute():
       ...       />
       ... </configure>'''
       >>> import Products.Five
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config("configure.zcml", Products.Five)
       >>> zcml.load_string(configure_zcml)
 
@@ -662,6 +694,3 @@ def test_suite():
     from Testing.ZopeTestCase import FunctionalDocTestSuite
     suite.addTest( FunctionalDocTestSuite() )
     return suite
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
