@@ -15,9 +15,8 @@
 
 Directives to emulate the 'http://namespaces.zope.org/browser'
 namespace in ZCML known from zope.app.
-
-$Id: metaconfigure.py 114789 2010-07-16 05:56:26Z davisagli $
 """
+
 import os
 from inspect import ismethod
 
@@ -33,10 +32,17 @@ from zope.publisher.interfaces.browser import IBrowserPublisher
 from zope.publisher.interfaces.browser import IBrowserRequest
 from zope.security.zcml import Permission
 
-import zope.app.publisher.browser.viewmeta
-from zope.app.publisher.browser.viewmeta import providesCallable
-from zope.app.publisher.browser.viewmeta import _handle_menu
-from zope.app.publisher.browser.viewmeta import _handle_for
+import zope.browserpage.metaconfigure
+from zope.browserpage.metaconfigure import providesCallable
+from zope.browserpage.metaconfigure import _handle_menu
+from zope.browserpage.metaconfigure import _handle_for
+from zope.browserpage.metadirectives import IViewDirective
+
+from AccessControl.class_init import InitializeClass
+from AccessControl.security import getSecurityInfo
+from AccessControl.security import protectClass
+from AccessControl.security import protectName
+from AccessControl.security import CheckerPrivateId
 
 from Products.Five.browser import BrowserView
 from Products.Five.browser.resource import FileResourceFactory
@@ -45,12 +51,7 @@ from Products.Five.browser.resource import PageTemplateResourceFactory
 from Products.Five.browser.resource import DirectoryResourceFactory
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from Products.Five.metaclass import makeClass
-from Products.Five.security import getSecurityInfo
-from Products.Five.security import protectClass
-from Products.Five.security import protectName
-from Products.Five.security import CheckerPrivateId
 
-from App.class_init import InitializeClass
 
 def page(_context, name, permission, for_,
          layer=IDefaultBrowserLayer, template=None, class_=None,
@@ -58,6 +59,7 @@ def page(_context, name, permission, for_,
          attribute='__call__', menu=None, title=None,
          ):
 
+    name = str(name)  # De-unicode
     _handle_menu(_context, menu, title, [for_], name, permission)
 
     if not (class_ or template):
@@ -165,7 +167,7 @@ def page(_context, name, permission, for_,
         args = (new_class,)
         )
 
-class pages(zope.app.publisher.browser.viewmeta.pages):
+class pages(zope.browserpage.metaconfigure.pages):
 
     def page(self, _context, name, attribute='__call__', template=None,
              menu=None, title=None):
@@ -178,8 +180,6 @@ class pages(zope.app.publisher.browser.viewmeta.pages):
 
 # view (named view with pages)
 
-from zope.app.publisher.browser.metadirectives import IViewDirective
-
 class IFiveViewDirective(IViewDirective):
 
     permission = Permission(
@@ -189,12 +189,14 @@ class IFiveViewDirective(IViewDirective):
         )
 
 
-class view(zope.app.publisher.browser.viewmeta.view):
+class view(zope.browserpage.metaconfigure.view):
 
     def __call__(self):
         (_context, name, for_, permission, layer, class_,
          allowed_interface, allowed_attributes) = self.args
 
+        name = str(name)  # De-unicode
+
         required = {}
 
         cdict = {}
@@ -443,7 +445,7 @@ def resourceDirectory(_context, name, directory, layer=IDefaultBrowserLayer,
             )
 
 class ViewMixinForAttributes(BrowserView,
-                             zope.app.publisher.browser.viewmeta.simple):
+                             zope.browserpage.metaconfigure.simple):
 
     # For some reason, the 'simple' baseclass doesn't implement this
     # mandatory method (see https://bugs.launchpad.net/zope3/+bug/129296)
