@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """Boiler plate test module
-
-$Id: boilerplate.py 110405 2010-04-01 16:13:15Z tseaver $
 """
 
 def test_boilerplate():
@@ -22,7 +20,7 @@ def test_boilerplate():
       >>> setUp()
 
       >>> import Products.Five.tests
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config('boilerplate.zcml', Products.Five.tests)
 
       >>> from Products.Five.tests.testing import manage_addFiveTraversableFolder
