@@ -14,6 +14,7 @@
 Test suite for session id manager.
 """
 import unittest
+import Testing
 
 class TestBrowserIdManager(unittest.TestCase):
 
@@ -642,6 +643,18 @@ class TestBrowserIdManagerTraverser(unittest.TestCase):
         self.assertEqual(request._script[1], bid)
 
 
+class TestBrowserIdManagerPublish(Testing.ZopeTestCase.FunctionalTestCase):
+
+    def test_encodeUrl_safe(self):
+        from OFS.Application import AppInitializer
+        init = AppInitializer(self.app)
+        init.install_browser_id_manager()
+
+        res = self.publish(
+            '/browser_id_manager/encodeUrl?url=%3Chtml%3EEVIL%2Fhtml%3E%3C!--')
+        self.assertNotIn("<html>EVIL/html>", res.getBody())
+
+
 class DummyObject:
     def __init__(self, **kw):
         self.__dict__.update(kw)
@@ -667,4 +680,5 @@ def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(TestBrowserIdManager),
         unittest.makeSuite(TestBrowserIdManagerTraverser),
+        unittest.makeSuite(TestBrowserIdManagerPublish),
     ))
