@@ -11,11 +11,8 @@
 #
 ##############################################################################
 """Provide conversion between Python pickles and XML
-
 """
 
-__version__ = "1.9"                     # Code version
-
 from pickle import *
 import struct
 import base64
@@ -598,102 +595,3 @@ class xmlPickler(NoBlanks, xyap):
         'persistent': save_persis,
         'unicode': save_unicode,
         }
-
-
-# The rest is used for testing only
-
-class C:
-    def __cmp__(self, other):
-        return cmp(self.__dict__, other.__dict__)
-
-def test():
-    import xmllib
-    import pickle
-    c = C()
-    c.foo = 1
-    c.bar = 2
-    x = [0, 1, 2, 3]
-    y = ('abc', 'abc', c, c)
-    x.append(y)
-    x.append(y)
-    t = ()
-    l = []
-    s = ''
-    L = long('999999999999')
-    x.append(t)
-    x.append(l)
-    x.append(s)
-    x.append(L)
-    x.append(55555)
-    x.append(13)
-    r = [x]
-    print x
-    f = pickle.dumps(x)
-    print f
-    r.append(f)
-    q = ToXMLloads(f)
-    q = str(q)
-    q = '<?xml version="1.0"?>\n' + q
-    print q
-    r.append(q)
-    F = xmlPickler()
-    F.binary = 1
-    p = xmllib.XMLParser()
-    p.start_handlers = F.start_handlers
-    p.end_handlers = F.end_handlers
-    p.handle_data = F.handle_data
-    p.unknown_starttag = F.unknown_starttag
-    p.unknown_endtag = F.unknown_endtag
-    p._stack = F._stack
-    p.push = F.push
-    p.append = F.append
-    data = q.split('\n')
-    for l in data:
-        p.feed(l)
-    p.close()
-    z = p._stack
-    z = z[0][0]
-    print z, '\n'
-    r.append(z)
-    l = pickle.loads(z)
-    print l, '\n'
-    r.append(l)
-
-def test1():
-    import xml.parsers.expat
-    import pickle
-    c = C()
-    c.foo = 1
-    c.bar = 2
-    x = [0, 1, 2, 3]
-    y = ('abc', 'abc', c, c)
-    x.append(y)
-    x.append(y)
-    t = ()
-    l = []
-    s = ''
-    L = long('999999999999')
-    x.append(t)
-    x.append(l)
-    x.append(s)
-    x.append(L)
-    x.append(5)
-    x.append(13)
-    print x, '\n'
-    f = pickle.dumps(x)
-    print f, '\n'
-    q = ToXMLloads(f)
-    q = str(q)
-    q = '<?xml version="1.0"?>\n' + q
-    print q, '\n'
-    F = xmlPickler()
-    F.binary = 0
-    p = xml.parsers.expat.ParserCreate()
-    p.CharacterDataHandler = F.handle_data
-    p.StartElementHandler = F.unknown_starttag
-    p.EndElementHandler = F.unknown_endtag
-    r = p.Parse(q)
-    print r, '\n'
-
-if __name__ == '__main__':
-    test()
