@@ -73,6 +73,9 @@ class BaseRequest_factory:
                 # Attribute without docstring.
                 return 'unpublishable'
 
+            def __contains__(self, name):
+                return False
+
         return DummyObjectBasic
 
     def _makeBasicObject(self):
@@ -124,6 +127,27 @@ class BaseRequest_factory:
                 return self, self._default_path
         return DummyObjectWithBD()
 
+    def _makeObjectWithBBT(self):
+        from ZPublisher.interfaces import UseTraversalDefault
+
+        class _DummyResult(object):
+            ''' '''
+            def __init__(self, tag):
+                self.tag = tag
+
+        class DummyObjectWithBBT(self._makeBasicObjectClass()):
+            """ Dummy class with __bobo_traverse__
+            """
+            default = _DummyResult('Default')
+
+            def __bobo_traverse__(self, REQUEST, name):
+                if name == 'normal':
+                    return _DummyResult('Normal')
+                elif name == 'default':
+                    raise UseTraversalDefault
+                raise AttributeError(name)
+        return DummyObjectWithBBT()
+
     def _makeObjectWithBDBBT(self):
         class DummyObjectWithBDBBT(self._makeBasicObjectClass()):
             """Dummy class with __browser_default__."""
@@ -173,6 +197,15 @@ class TestBaseRequest(unittest.TestCase, BaseRequest_factory):
         self.assertEqual(r.URL, '/folder/objBasic')
         self.assertEqual(r.response.base, '')
 
+    def test_traverse_basic_no_acquire_flag(self):
+        root, folder = self._makeRootAndFolder()
+        folder._setObject('objBasic', self._makeBasicObject())
+        r = self._makeOne(root)
+        r['REQUEST_METHOD'] = 'HEAD'
+        r.traverse('folder/objBasic')
+        self.assertEqual(r.URL, '/folder/objBasic')
+        self.assertEqual(r.response.base, '')
+
     def test_traverse_withDefault(self):
         root, folder = self._makeRootAndFolder()
         folder._setObject('objWithDefault', self._makeObjectWithDefault())
@@ -239,7 +272,7 @@ class TestBaseRequest(unittest.TestCase, BaseRequest_factory):
         folder._setObject('objWithBD', self._makeObjectWithBD())
         folder.objWithBD._default_path = ['']
         r = self._makeOne(root)
-        self.failUnlessRaises(NotFound, r.traverse, 'folder/objWithBD')
+        self.assertRaises(NotFound, r.traverse, 'folder/objWithBD')
 
     def test_traverse_withBBT_handles_AttributeError(self):
         # Test that if __bobo_traverse__ raises AttributeError
@@ -252,9 +285,19 @@ class TestBaseRequest(unittest.TestCase, BaseRequest_factory):
         obj.__bobo_traverse__ = _faux___bobo_traverse__
         folder._setObject('objWithBBT', obj)
         r = self._makeOne(root)
-        self.failUnlessRaises(NotFound, r.traverse,
+        self.assertRaises(NotFound, r.traverse,
                               'folder/objWithBBT/bbt_foo')
 
+    def test_traverse_UseTraversalDefault(self):
+        root, folder = self._makeRootAndFolder()
+        folder._setObject('objWithBBT', self._makeObjectWithBBT())
+        # test non default usage
+        r = self._makeOne(root)
+        self.assertEqual(r.traverse('folder/objWithBBT/normal').tag, 'Normal')
+        # test default usage
+        r = self._makeOne(root)
+        self.assertEqual(r.traverse('folder/objWithBBT/default').tag, 'Default')
+
     def test_traverse_withBDBBT(self):
         # Test for an object which has a __browser_default__
         # and __bobo_traverse__
@@ -278,7 +321,7 @@ class TestBaseRequest(unittest.TestCase, BaseRequest_factory):
         folder._setObject('objWithBDBBT', self._makeObjectWithBDBBT())
         folder.objWithBDBBT._default_path = ['xxx']
         r = self._makeOne(root)
-        self.failUnlessRaises(NotFound, r.traverse, 'folder/objWithBDBBT')
+        self.assertRaises(NotFound, r.traverse, 'folder/objWithBDBBT')
 
     def test_traverse_slash(self):
         root, folder = self._makeRootAndFolder()
@@ -682,6 +725,3 @@ def test_suite():
         unittest.makeSuite(TestBaseRequest),
         unittest.makeSuite(TestBaseRequestZope3Views),
         ))
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
