@@ -11,12 +11,11 @@
 #
 ##############################################################################
 """WebDAV support - null resource objects.
-
-$Id: NullResource.py 113513 2010-06-16 10:04:54Z jens $
 """
 
 import sys
 
+from AccessControl.class_init import InitializeClass
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from AccessControl.SecurityManagement import getSecurityManager
 from AccessControl.Permissions import view as View
@@ -26,7 +25,6 @@ from AccessControl.Permissions import webdav_unlock_items
 from Acquisition import aq_base
 from Acquisition import aq_parent
 from Acquisition import Implicit
-from App.class_init import InitializeClass
 from App.special_dtml import DTMLFile
 from Persistence import Persistent
 from OFS.CopySupport import CopyError
