@@ -3,7 +3,7 @@ import unittest
 
 class TestEtagSupport(unittest.TestCase):
 
-    def test_z3interfaces(self):
+    def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from webdav.EtagSupport import EtagBaseInterface
         from webdav.EtagSupport import EtagSupport
@@ -15,6 +15,3 @@ def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(TestEtagSupport),
         ))
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
