@@ -35,16 +35,13 @@ TODO
 
     add in tests for objects which are modified multiple times,
     for example an object that gets modified in multiple sub txns.
-
-$Id: test_transaction.py 112144 2010-05-07 15:35:15Z tseaver $
 """
+from doctest import DocTestSuite, DocFileSuite
 
 import struct
+import sys
 import unittest
-import warnings
-
 import transaction
-from transaction.tests.warnhook import WarningsHook
 
 _ADDRESS_MASK = 256 ** struct.calcsize('P')
 def positive_id(obj):
@@ -245,6 +242,25 @@ class TransactionTests(unittest.TestCase):
 ##            self.fail("Hosed Application didn't stop commits")
 
 
+class Test_oid_repr(unittest.TestCase):
+    def _callFUT(self, oid):
+        from transaction._transaction import oid_repr
+        return oid_repr(oid)
+
+    def test_as_nonstring(self):
+        self.assertEqual(self._callFUT(123), '123')
+
+    def test_as_string_not_8_chars(self):
+        self.assertEqual(self._callFUT('a'), "'a'")
+
+    def test_as_string_z64(self):
+        s = '\0'*8
+        self.assertEqual(self._callFUT(s), '0x00')
+
+    def test_as_string_all_Fs(self):
+        s = '\1'*8
+        self.assertEqual(self._callFUT(s), '0x0101010101010101')
+
 class DataObject:
 
     def __init__(self, transaction_manager, nost=0):
@@ -673,8 +689,8 @@ def test_addAfterCommitHook():
       >>> do = DataObject(mgr)
 
       >>> t = transaction.begin()
-      >>> len(t._manager._txns)
-      1
+      >>> t._manager._txn is not None
+      True
 
       >>> t.addAfterCommitHook(hook, ('-', 1))
       >>> transaction.commit()
@@ -682,19 +698,78 @@ def test_addAfterCommitHook():
       >>> log
       ["True arg '-' kw1 1 kw2 'no_kw2'"]
 
-      >>> len(t._manager._txns)
-      0
+      >>> t._manager._txn is not None
+      False
 
       >>> reset_log()
     """
 
+def bug239086():
+    """
+    The original implementation of thread transaction manager made
+    invalid assumptions about thread ids.
+
+    >>> import transaction.tests.savepointsample
+    >>> dm = transaction.tests.savepointsample.SampleSavepointDataManager()
+    >>> dm.keys()
+    []
+
+    >>> class Sync:
+    ...      def __init__(self, label):
+    ...          self.label = label
+    ...      def beforeCompletion(self, t):
+    ...          print self.label, 'before'
+    ...      def afterCompletion(self, t):
+    ...          print self.label, 'after'
+    ...      def newTransaction(self, t):
+    ...          print self.label, 'new'
+    >>> sync = Sync(1)
+
+    >>> import threading
+    >>> def run_in_thread(f):
+    ...     t = threading.Thread(target=f)
+    ...     t.start()
+    ...     t.join()
+
+    >>> @run_in_thread
+    ... def first():
+    ...     transaction.manager.registerSynch(sync)
+    ...     transaction.manager.begin()
+    ...     dm['a'] = 1
+    1 new
+
+    >>> @run_in_thread
+    ... def second():
+    ...     transaction.abort() # should do nothing.
+
+    >>> dm.keys()
+    ['a']
+
+    >>> dm = transaction.tests.savepointsample.SampleSavepointDataManager()
+    >>> dm.keys()
+    []
+
+    >>> @run_in_thread
+    ... def first():
+    ...     dm['a'] = 1
+
+    >>> transaction.abort() # should do nothing
+    >>> dm.keys()
+    ['a']
+
+    """
+
 def test_suite():
-    from doctest import DocTestSuite, DocFileSuite
-    return unittest.TestSuite((
+    suite = unittest.TestSuite((
         DocFileSuite('doom.txt'),
         DocTestSuite(),
         unittest.makeSuite(TransactionTests),
+        unittest.makeSuite(Test_oid_repr),
         ))
+    if sys.version_info >= (2, 6):
+        suite.addTest(DocFileSuite('convenience.txt'))
+
+    return suite
 
 # additional_tests is for setuptools "setup.py test" support
 additional_tests = test_suite
