@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002, 2009 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002, 2009 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,19 +12,21 @@
 #
 ##############################################################################
 """Component Architecture Tests
-
-$Id: tests.py 102214 2009-07-24 08:56:56Z srichter $
 """
+
+import doctest
+import persistent
 import re
+import sys
 import unittest
 import transaction
-import persistent
 from cStringIO import StringIO
 
 from zope import interface, component
 from zope.interface.verify import verifyObject
 from zope.interface.interfaces import IInterface
-from zope.testing import doctest, renormalizing
+from zope.testing import renormalizing
+from zope.testing.testrunner.layer import UnitTests
 
 from zope.component.interfaces import ComponentLookupError
 from zope.component.interfaces import IComponentArchitecture
@@ -1088,6 +1090,28 @@ def dont_leak_utility_registrations_in__subscribers():
 
     """
 
+def test_zcml_handler_site_manager():
+    """
+    The ZCML directives provided by zope.component use the ``getSiteManager``
+    method to get the registry where to register the components. This makes
+    possible to hook ``getSiteManager`` before loading a ZCML file:
+
+    >>> from zope.component.registry import Components
+    >>> registry = Components()
+    >>> def dummy(context=None):
+    ...     return registry
+    >>> from zope.component import getSiteManager
+    >>> ignore = getSiteManager.sethook(dummy)
+
+    >>> from zope.component.testfiles.components import comp, IApp
+    >>> from zope.component.zcml import handler
+    >>> handler('registerUtility', comp, IApp, u'')
+    >>> registry.getUtility(IApp) is comp
+    True
+    >>> ignore = getSiteManager.reset()
+
+    """
+
 class StandaloneTests(unittest.TestCase):
     def testStandalone(self):
         import subprocess
@@ -1635,6 +1659,28 @@ class ResourceViewTests(PlacelessSetup, unittest.TestCase):
         self.assertRaises(ValueError, xmlconfig, config, testing=1)
 
 
+class ConditionalSecurityLayer(UnitTests):
+
+    __name__ = 'ConditionalSecurity'
+    __bases__ = ()
+
+    def setUp(self):
+        setUp()
+        self.modules = {}
+        for m in ('zope.security', 'zope.proxy'):
+            self.modules[m] = sys.modules[m]
+            sys.modules[m] = None
+        import zope.component.zcml
+        reload(zope.component.zcml)
+
+    def tearDown(self):
+        tearDown()
+        for m in ('zope.security', 'zope.proxy'):
+            sys.modules[m] = self.modules[m]
+        import zope.component.zcml
+        reload(zope.component.zcml)
+
+
 def setUpRegistryTests(tests):
     setUp()
 
@@ -1655,6 +1701,12 @@ def test_suite():
                     r'exceptions.\1Error:'),
         ])
 
+    zcml_conditional = doctest.DocFileSuite('zcml_conditional.txt', checker=checker)
+    zcml_conditional.layer = ConditionalSecurityLayer()
+
+    hooks_conditional = doctest.DocFileSuite('hooks.txt', checker=checker)
+    hooks_conditional.layer = ConditionalSecurityLayer()
+
     return unittest.TestSuite((
         doctest.DocTestSuite(setUp=setUp, tearDown=tearDown),
         unittest.makeSuite(HookableTests),
@@ -1670,10 +1722,19 @@ def test_suite():
         doctest.DocFileSuite('registry.txt', checker=checker,
                              setUp=setUpRegistryTests,
                              tearDown=tearDownRegistryTests),
+        doctest.DocFileSuite('hooks.txt',checker=checker,
+                             setUp=setUp, tearDown=tearDown),
         doctest.DocFileSuite('event.txt',
                              setUp=setUp, tearDown=tearDown),
-        doctest.DocFileSuite('zcml.txt',checker=checker,
+        doctest.DocTestSuite('zope.component.security'),
+        doctest.DocFileSuite('zcml.txt', checker=checker,
                              setUp=setUp, tearDown=tearDown),
+        doctest.DocFileSuite('testlayer.txt',
+                             optionflags=(doctest.ELLIPSIS +
+                                          doctest.NORMALIZE_WHITESPACE +
+                                          doctest.REPORT_NDIFF)),
+        zcml_conditional,
+        hooks_conditional,
         unittest.makeSuite(StandaloneTests),
         unittest.makeSuite(ResourceViewTests),
         ))
