@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,14 +12,12 @@
 #
 ##############################################################################
 """Test base proxy class.
-
-$Id: test_proxy.py 87706 2008-06-24 12:23:50Z srichter $
 """
+from doctest import DocTestSuite
 import pickle
 import sys
 import unittest
 
-from zope.testing.doctestunit import DocTestSuite
 from zope.proxy import ProxyBase
 import zope.proxy
 
@@ -509,7 +507,7 @@ def test_queryProxy():
     1
     >>> int(queryProxy(p2, ProxyBase) is p2)
     1
-    
+
     """
 
 def test_queryInnerProxy():
@@ -546,7 +544,7 @@ def test_queryInnerProxy():
     1
     >>> int(queryInnerProxy(p3, P2) is p2)
     1
-    
+
     """
 
 def test_sameProxiedObjects():
@@ -601,7 +599,7 @@ def test_subclassing_proxies():
     >>> p = MyProxy(l)
 
     I can use attributes defined by the class, including slots:
-    
+
     >>> p.x = 'x'
     >>> p.x
     'x'
@@ -609,14 +607,14 @@ def test_subclassing_proxies():
     'x'
 
     I can also use attributes of the proxied object:
-    
+
     >>> p
     [1, 2, 3]
     >>> p.pop()
     3
     >>> p
     [1, 2]
-    
+
     """
 
 def test_get_descriptors_in_proxy_class():
@@ -648,7 +646,7 @@ def test_get_descriptors_in_proxy_class():
     >>> p.z = 1
     >>> o.z, p.z
     (1, 1)
-    
+
     """
 
 def test_non_overridable():
@@ -661,7 +659,7 @@ def test_non_overridable():
     defined in proxy classes take precedence over functions defined
     in proxied objects.
 
-    
+
     >>> class MyProxy(ProxyBase):
     ...    __slots__ = ()
     ...
@@ -677,7 +675,7 @@ def test_non_overridable():
     >>> p = MyProxy(o)
     >>> p.foo()
     'MyProxy foo'
-    
+
     """
 
 def test_setProxiedObject():
@@ -706,10 +704,13 @@ def test_setProxiedObject():
     The first argument  to `setProxiedObject()` must be a proxy; other objects
     cause it to raise an exception:
 
-    >>> setProxiedObject(c1, None)
-    Traceback (most recent call last):
-    TypeError: setProxiedObject() argument 1 must be zope.proxy.ProxyBase, not C
-
+    >>> try:
+    ...     setProxiedObject(c1, None)
+    ... except TypeError:
+    ...     print "TypeError raised"
+    ... else:
+    ...     print "Expected TypeError not raised"
+    TypeError raised
     """
 
 def test_suite():
