@@ -13,10 +13,12 @@
 ##############################################################################
 """Container constraint tests
 
-$Id: test_constraints.py 95341 2009-01-28 15:59:18Z wosc $
+$Id: test_constraints.py 111713 2010-04-30 20:43:09Z hannosch $
 """
+
+import doctest
 import unittest
-from zope.testing import doctest, module
+from zope.testing import module
 
 def setUp(test):
     module.setUp(test, 'zope.container.constraints_txt')
