@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001-2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,18 +12,15 @@
 #
 ##############################################################################
 """Interface object implementation
-
-$Id: interface.py 105433 2009-11-02 08:33:25Z ctheune $
 """
-
 from __future__ import generators
 
 import sys
-import warnings
-import weakref
 from types import FunctionType
-from ro import ro
+import weakref
+
 from zope.interface.exceptions import Invalid
+from zope.interface.ro import ro
 
 
 CO_VARARGS = 4
@@ -163,6 +160,10 @@ class SpecificationBasePy(object):
     __call__ = isOrExtends
 
 SpecificationBase = SpecificationBasePy
+try:
+    from _zope_interface_coptimizations import SpecificationBase
+except ImportError:
+    pass
 
 _marker = object()
 class InterfaceBasePy(object):
@@ -197,18 +198,21 @@ class InterfaceBasePy(object):
             adapter = hook(self, obj)
             if adapter is not None:
                 return adapter
+
     
 InterfaceBase = InterfaceBasePy
-
-adapter_hooks = []
-
 try:
-    import _zope_interface_coptimizations
+    from _zope_interface_coptimizations import InterfaceBase
 except ImportError:
     pass
-else:
-    from _zope_interface_coptimizations import SpecificationBase
-    from _zope_interface_coptimizations import InterfaceBase, adapter_hooks
+
+
+adapter_hooks = []
+try:
+    from _zope_interface_coptimizations import adapter_hooks
+except ImportError:
+    pass
+
 
 class Specification(SpecificationBase):
     """Specifications
@@ -293,6 +297,10 @@ class Specification(SpecificationBase):
     def changed(self, originally_changed):
         """We, or something we depend on, have changed
         """
+        try:
+            del self._v_attrs
+        except AttributeError:
+            pass
 
         implied = self._implied
         implied.clear()
@@ -336,12 +344,8 @@ class Specification(SpecificationBase):
           >>> spec = Specification((I2, I3))
           >>> spec = Specification((I4, spec))
           >>> i = spec.interfaces()
-          >>> i.next().getName()
-          'I4'
-          >>> i.next().getName()
-          'I2'
-          >>> i.next().getName()
-          'I3'
+          >>> [x.getName() for x in i]
+          ['I4', 'I2', 'I3']
           >>> list(i)
           []
         """
@@ -476,6 +480,9 @@ class InterfaceClass(Element, InterfaceBase, Specification):
         # Make sure that all recorded attributes (and methods) are of type
         # `Attribute` and `Method`
         for name, attr in attrs.items():
+            if name == '__locals__':
+                # This happens under Python 3 sometimes, not sure why. /regebro
+                continue
             if isinstance(attr, Attribute):
                 attr.interface = self
                 if not attr.__name__:
@@ -501,8 +508,8 @@ class InterfaceClass(Element, InterfaceBase, Specification):
           ...
           >>>
           >>> i = I1.interfaces()
-          >>> i.next().getName()
-          'I1'
+          >>> [x.getName() for x in i]
+          ['I1']
           >>> list(i)
           []
         """
@@ -570,7 +577,7 @@ class InterfaceClass(Element, InterfaceBase, Specification):
         exec "class %s: pass" % self.__name__ in klass
         klass=klass[self.__name__]
 
-        self.__d(klass.__dict__)
+        self.__d(klass)
 
         self._deferred=klass
 
@@ -599,14 +606,13 @@ class InterfaceClass(Element, InterfaceBase, Specification):
         """Retrieve a named interface."""
         return None
 
-    def __d(self, dict):
-
+    def __d(self, klass):
         for k, v in self.__attrs.items():
-            if isinstance(v, Method) and not (k in dict):
-                dict[k]=v
+            if isinstance(v, Method) and not (k in klass.__dict__):
+                setattr(klass, k, v)
 
         for b in self.__bases__:
-            b.__d(dict)
+            b.__d(klass)
 
     def __repr__(self):
         try:
@@ -672,18 +678,26 @@ class InterfaceClass(Element, InterfaceBase, Specification):
         if o2 is None:
             return -1
 
-        n1 = (getattr(o1, '__name__', ''),
-              getattr(getattr(o1,  '__module__', None), '__name__', ''))
-        n2 = (getattr(o2, '__name__', ''),
-              getattr(getattr(o2,  '__module__', None), '__name__', ''))
+        n1 = (getattr(o1, '__name__', ''), getattr(o1,  '__module__', None))
+        n2 = (getattr(o2, '__name__', ''), getattr(o2,  '__module__', None))
 
-        return cmp(n1, n2)
+        return (n1 > n2) - (n1 < n2)
 
     def __lt__(self, other):
         c = self.__cmp(self, other)
         #print '<', self, other, c < 0, c
         return c < 0
 
+    def __le__(self, other):
+        c = self.__cmp(self, other)
+        #print '<', self, other, c < 0, c
+        return c <= 0
+
+    def __ge__(self, other):
+        c = self.__cmp(self, other)
+        #print '>', self, other, c > 0, c
+        return c >= 0
+
     def __gt__(self, other):
         c = self.__cmp(self, other)
         #print '>', self, other, c > 0, c
@@ -801,11 +815,14 @@ def _wire():
     from zope.interface.interfaces import IMethod
     classImplements(Method, IMethod)
 
-    from zope.interface.interfaces import IInterface, ISpecification
+    from zope.interface.interfaces import IInterface
     classImplements(InterfaceClass, IInterface)
+
+    from zope.interface.interfaces import ISpecification
     classImplements(Specification, ISpecification)
 
 # We import this here to deal with module dependencies.
-from zope.interface.declarations import providedBy, implementedBy
+from zope.interface.declarations import implementedBy
+from zope.interface.declarations import providedBy
 from zope.interface.exceptions import InvalidInterface
 from zope.interface.exceptions import BrokenImplementation
