@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """Virtual File System interfaces for the publisher.
-
-$Id: ftp.py 41055 2005-12-29 22:29:00Z hdima $
 """
 
 __docformat__ = "reStructuredText"
@@ -22,12 +20,15 @@ from zope.interface import Interface
 
 from zope.publisher.interfaces import IPublishTraverse
 from zope.publisher.interfaces import IRequest
-
+from zope.publisher.interfaces import IView
 
 class IFTPRequest(IRequest):
     """FTP Request
     """
 
+class IFTPView(IView):
+    """FTP View"""
+
 class IFTPCredentials(Interface):
 
     def _authUserPW():
