@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """HTTP-related publisher interfaces.
-
-$Id: http.py 100272 2009-05-23 02:07:17Z shane $
 """
 
 __docformat__ = "reStructuredText"
@@ -27,6 +25,7 @@ from zope.publisher.interfaces import IApplicationRequest
 from zope.publisher.interfaces import IPublishTraverse
 from zope.publisher.interfaces import IRequest
 from zope.publisher.interfaces import IResponse
+from zope.publisher.interfaces import IView
 
 
 class IVirtualHostRequest(Interface):
@@ -214,6 +213,10 @@ class IHTTPRequest(IRequest):
         """
 
 
+class IHTTPView(IView):
+    "HTTP View"
+
+
 class IHTTPCredentials(Interface):
 
     # TODO: Eventially this will be a different method
@@ -227,11 +230,17 @@ class IHTTPCredentials(Interface):
 
 
 class IHTTPApplicationResponse(Interface):
-    """HTTP Response
-    """
+    """HTTP Response"""
 
-    def redirect(location, status=302):
+    def redirect(location, status=302, trusted=False):
         """Causes a redirection without raising an error.
+
+        By default redirects are untrusted which restricts target URLs to the
+        same host that the request was sent to.
+
+        If the `trusted` flag is set, redirects are allowed for any target
+        URL.
+
         """
 
 class IHeaderOutput(Interface):
