@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -11,16 +11,17 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""An adapter from zope.security.interfaces.IPrincipal to
-zope.publisher.interfaces.logginginfo.ILoggingInfo.
+"""Adapter:
 
-$Id: principallogging.py 97915 2009-03-11 21:07:13Z nadako $
+Adapts zope.security.interfaces.IPrincipal to
+zope.publisher.interfaces.logginginfo.ILoggingInfo.
 """
 from zope.component import adapts
 from zope.interface import implements
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
 from zope.security.interfaces import IPrincipal
 
+
 class PrincipalLogging(object):
 
     adapts(IPrincipal)
@@ -30,4 +31,4 @@ class PrincipalLogging(object):
         self.principal = principal
 
     def getLogMessage(self):
-        return str(self.principal.id)
+        return self.principal.id.encode('ascii', 'backslashreplace')
