@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 Zope Corporation and Contributors.
+# Copyright (c) 2004 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,11 +12,9 @@
 #
 ##############################################################################
 """Test zope.publisher.browser doctests
-
-$Id: test_browser.py 97600 2009-03-06 19:58:44Z gary $
 """
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+from doctest import DocTestSuite
 
 __docformat__ = "reStructuredText"
 
