@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 Zope Corporation and Contributors.
+# Copyright (c) 2004 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,12 +12,12 @@
 #
 ##############################################################################
 """Tests for the schema package's documentation files
-
-$Id: test_docs.py 79527 2007-09-07 18:31:24Z nikhil_n $
 """
-import unittest
+import doctest
 import re
-from zope.testing import doctest, renormalizing
+import unittest
+
+from zope.testing import renormalizing
 
 def test_suite():
     checker = renormalizing.RENormalizing([
@@ -32,6 +32,3 @@ def test_suite():
             '../validation.txt', checker=checker,
             optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS),
         ))
-
-if __name__ == '__main__':
-    unittest.main(default='test_suite')
