@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Corporation and Contributors.
+# Copyright (c) 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,22 +12,16 @@
 #
 ##############################################################################
 """Mechanize-based Functional Doctest interfaces
-
-$Id: browser.py 94307 2008-12-24 14:47:46Z gary $
 """
 
 __docformat__ = "reStructuredText"
 
 
 import cStringIO
-import datetime
-import operator
 import re
 import sys
 import time
-import urllib2
 
-import ClientForm
 import mechanize
 import zope.interface
 
@@ -43,7 +37,7 @@ def disambiguate(intermediate, msg, index):
     if intermediate:
         if index is None:
             if len(intermediate) > 1:
-                raise ClientForm.AmbiguityError(msg)
+                raise mechanize.AmbiguityError(msg)
             else:
                 return intermediate[0]
         else:
@@ -54,7 +48,7 @@ def disambiguate(intermediate, msg, index):
     raise LookupError(msg)
 
 def controlFactory(control, form, browser):
-    if isinstance(control, ClientForm.Item):
+    if isinstance(control, mechanize.Item):
         # it is a subcontrol
         return ItemControl(control, form, browser)
     else:
@@ -198,6 +192,8 @@ class Browser(SetattrErrorsMixin):
         if self._contents is not None:
             return self._contents
         response = self.mech_browser.response()
+        if response is None:
+            return None
         old_location = response.tell()
         response.seek(0)
         self._contents = response.read()
@@ -216,16 +212,21 @@ class Browser(SetattrErrorsMixin):
 
         def get(self):
             headers = self.mech_browser.addheaders
-            return dict(headers).get(header_key, True)
+            value = dict(headers).get(header_key, True)
+            return {'False': False}.get(value, True)
 
         def set(self, value):
             headers = self.mech_browser.addheaders
             current_value = get(self)
             if current_value == value:
                 return
-            if header_key in dict(headers):
-                headers.remove((header_key, current_value))
-            headers.append((header_key, value))
+
+            # Remove the current header...
+            for key, header_value in headers[:]:
+                if key == header_key:
+                    headers.remove((key, header_value))
+            # ... Before adding the new one.
+            headers.append((header_key, {False: 'False'}.get(value, 'True')))
 
         return property(get, set)
 
@@ -240,7 +241,7 @@ class Browser(SetattrErrorsMixin):
                 except Exception, e:
                     fix_exception_name(e)
                     raise
-            except urllib2.HTTPError, e:
+            except mechanize.HTTPError, e:
                 if e.code >= 200 and e.code <= 299:
                     # 200s aren't really errors
                     pass
@@ -255,7 +256,7 @@ class Browser(SetattrErrorsMixin):
             code, msg = self.headers['Status'].split(' ', 1)
             code = int(code)
             if self.raiseHttpErrors and code >= 400:
-                raise urllib2.HTTPError(url, code, msg, self.headers, fp=None)
+                raise mechanize.HTTPError(url, code, msg, self.headers, fp=None)
 
     def post(self, url, data, content_type=None):
         if content_type is not None:
@@ -326,6 +327,10 @@ class Browser(SetattrErrorsMixin):
         args['nr'] = index
         return Link(self.mech_browser.find_link(**args), self)
 
+    def follow(self, *args, **kw):
+        """Select a link and follow it."""
+        self.getLink(*args, **kw).click()
+
     def _findByLabel(self, label, forms, include_subcontrols=False):
         # forms are iterable of mech_forms
         matches = re.compile(r'(^|\b|\W)%s(\b|\W|$)'
@@ -405,14 +410,17 @@ class Browser(SetattrErrorsMixin):
             label = labels[0].text
         else:
             label = None
-        self._start_timer()
         try:
-            self.mech_browser.open(form.click(
-                id=control.id, name=control.name, label=label, coord=coord))
-        except Exception, e:
-            fix_exception_name(e)
-            raise
-        self._stop_timer()
+            self._start_timer()
+            try:
+                self.mech_browser.form = form
+                self.mech_browser.submit(id=control.id, name=control.name,
+                                         label=label, coord=coord)
+            except Exception, e:
+                fix_exception_name(e)
+                raise
+        finally:
+            self._stop_timer()
 
     def _changed(self):
         self._counter += 1
@@ -432,9 +440,15 @@ class Link(SetattrErrorsMixin):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
         self.browser._start_timer()
-        self.browser.mech_browser.follow_link(self.mech_link)
-        self.browser._stop_timer()
-        self.browser._changed()
+        try:
+            try:
+                self.browser.mech_browser.follow_link(self.mech_link)
+            except Exception, e:
+                fix_exception_name(e)
+                raise
+        finally:
+            self.browser._stop_timer()
+            self.browser._changed()
 
     @property
     def url(self):
@@ -473,7 +487,7 @@ class Control(SetattrErrorsMixin):
             self.filename = None
             self.content_type = None
 
-        # for some reason ClientForm thinks we shouldn't be able to modify
+        # for some reason mechanize thinks we shouldn't be able to modify
         # hidden fields, but while testing it is sometimes very important
         if self.mech_control.type == 'hidden':
             self.mech_control.readonly = False
@@ -546,7 +560,7 @@ class ListControl(Control):
     def displayValue():
         """See zope.testbrowser.interfaces.IListControl"""
         # not implemented for anything other than select;
-        # would be nice if ClientForm implemented for checkbox and radio.
+        # would be nice if mechanize implemented for checkbox and radio.
         # attribute error for all others.
 
         def fget(self):
@@ -621,8 +635,10 @@ class SubmitControl(Control):
     def click(self):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
-        self.browser._clickSubmit(self.mech_form, self.mech_control, (1,1))
-        self.browser._changed()
+        try:
+            self.browser._clickSubmit(self.mech_form, self.mech_control, (1,1))
+        finally:
+            self.browser._changed()
 
 
 class ImageControl(Control):
@@ -631,8 +647,10 @@ class ImageControl(Control):
     def click(self, coord=(1,1)):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
-        self.browser._clickSubmit(self.mech_form, self.mech_control, coord)
-        self.browser._changed()
+        try:
+            self.browser._clickSubmit(self.mech_form, self.mech_control, coord)
+        finally:
+            self.browser._changed()
 
 
 class ItemControl(SetattrErrorsMixin):
@@ -695,7 +713,7 @@ class Form(SetattrErrorsMixin):
         """Initialize the Form
 
         browser - a Browser instance
-        form - a ClientForm instance
+        form - a mechanize.HTMLForm instance
         """
         self.browser = browser
         self.mech_form = form
@@ -728,23 +746,31 @@ class Form(SetattrErrorsMixin):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
         form = self.mech_form
-        if label is not None or name is not None:
-            intermediate, msg = self.browser._get_all_controls(
-                label, name, (form,))
-            intermediate = [
-                (control, form) for (control, form) in intermediate if
-                control.type in ('submit', 'submitbutton', 'image')]
-            control, form = disambiguate(intermediate, msg, index)
-            self.browser._clickSubmit(form, control, coord)
-        else: # JavaScript sort of submit
-            if index is not None or coord != (1,1):
-                raise ValueError(
-                    'May not use index or coord without a control')
-            request = self.mech_form._switch_click("request", urllib2.Request)
-            self.browser._start_timer()
-            self.browser.mech_browser.open(request)
-            self.browser._stop_timer()
-        self.browser._changed()
+        try:
+            if label is not None or name is not None:
+                intermediate, msg = self.browser._get_all_controls(
+                    label, name, (form,))
+                intermediate = [
+                    (control, form) for (control, form) in intermediate if
+                    control.type in ('submit', 'submitbutton', 'image')]
+                control, form = disambiguate(intermediate, msg, index)
+                self.browser._clickSubmit(form, control, coord)
+            else: # JavaScript sort of submit
+                if index is not None or coord != (1,1):
+                    raise ValueError(
+                        'May not use index or coord without a control')
+                request = self.mech_form._switch_click("request", mechanize.Request)
+                self.browser._start_timer()
+                try:
+                    try:
+                        self.browser.mech_browser.open(request)
+                    except Exception, e:
+                        fix_exception_name(e)
+                        raise
+                finally:
+                    self.browser._stop_timer()
+        finally:
+            self.browser._changed()
 
     def getControl(self, label=None, name=None, index=None):
         """See zope.testbrowser.interfaces.IBrowser"""
